#    most of this task is now taken from the standard application

set VERSION "FAIR"

#    put some default environment variables before loading common 

if {[info exists env(RDOUThdwre)] == 0} then {set env(RDOUThdwre) VME} 

loadsource  sys.common

set frametitle "MIDAS Data Acquisition Experiment Control (TUDA)"

set VMEPresent 1
set VMEEnable 1

set TSEnable 0

set histEnable 1
set HistMode 1

set txEnable 1

set TestBuffer ""

#    Group bases for now
set S9418_Base  1
set V767_Base   21

#    some additional registers for the FAIR system
lappend registerlist [list $RDOUThdwre.Verbose1   generic  "-nVerbose1 -tint"]
lappend registerlist [list $RDOUThdwre.Verbose2   generic  "-nVerbose2 -tint"]
lappend registerlist [list $RDOUThdwre.Verbose3   generic  "-nVerbose3 -tint"]


lappend actionlist [list "Show histogram name translations" "Show_Xlate"]
lappend actionlist [list "Edit histogram translation file"  "edit_xlate"]


proc SpectrumName {n} {Give_Xlate $n}

proc create_spectra {} {
   global SpectrumServer HistMode EGreport
   global V767_Base S9418_Base

   if {$HistMode == 0} {return}

   set-footer "Creating spectra"

   set EGreport_SAVE $EGreport
   set EGreport -1
   
   Make_Xlate $SpectrumServer

   set range [expr (($V767_Base + 4) * 64) + 64]
   
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Stat' !new)" } m]
   map_spectrum "Stat"

   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Rate' !new)" } m]

#   for silena 9418

   set maxmodule [inform fsysc inquire NumADCs]
   set maxadc 32

   for {set i 1} {$i <= $maxmodule} {incr i} {
    for {set j 0} {$j < $maxadc} {incr j} {

      set spectrumname [SpectrumName s9418#$i.adc$j]
      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 4096 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum s9418#$i.adc$j"
          clear-footer
          error $m
      } else {
          set group [expr $S9418_Base - 1 + $i]
          map_spectrum  "$spectrumname $group $j"
      }
    }
   }
   
#    for CAEN V767

   set maxmodule [inform vmecontrol inquire v767count]
   set maxadc 128

   for {set i 1} {$i <= $maxmodule} {incr i} {
    for {set j 0} {$j < $maxadc} {incr j} {

      set spectrumname [SpectrumName v767#$i.tdc$j]
      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 1024 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum v767#$i.tdc$j"
          clear-footer
          error $m
      } else {
         if {$j < 64} then {
             set group [expr $V767_Base + (($i - 1) * 2)]
             set item $j
         } else {
             set group [expr $V767_Base + (($i - 1) * 2) + 1]
             set item [expr $j - 64]
         }
          map_spectrum  "$spectrumname $group $item"
      }
    }
   }
   
   set EGreport $EGreport_SAVE
}

proc define_VME_addresses {} {
    global VMEserver
    global S9418_Base V767_Base

#    load hardware configuration into readout software

   ClearConfig

#    define group bases for data 
    DefineConfig "767  0 [format "0x%x" $V767_Base]"

#    the V767 VME modules

   set maxmodule [inform vmecontrol inquire v767count]

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {eg "(register :name '/$VMEserver/v767#$i.Output_Buffer'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register v767#$i.Output_Buffer"
          clear-footer
          error $m
      }
      DefineConfig "767 $i [string range [first $m] 2 end]"
   }

#    then the FAIR System Controller module 

   set z [catch {eg "(register :name '/$VMEserver/fsysc#1.RAM'  ?object)" } m]
   if {$z == 0} then {
        DefineConfig "0 0 [string range [first $m] 2 end]";
   }
}

proc DefineConfig {s} {

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return}

#    format of module configuration record is  type  module_number  VME_address
#    where type =2 (SYSC module)

   write-register EXEC.Parameter $s
   write-register EXEC.Action 13
   write-register EXEC.Sig1 0
   wait_for_completion "Define Configuration"
}

proc click-vme {action} {

   starttask vmecontrol

#   if {$action == 0} then {inform vmecontrol SAC_STOP}

    switch $action {
      0    {set p DAQ-halt}
      1    {set p DAQ-go}
      2    {set p DAQ-reset}
      3    {set p DAQ-setup}
    }

   set-busy
 
   set-footer "sending $p to VME control task"
   set z [catch {send vmecontrol $p} m]

   clear-busy
   if {$m == ""} then {clear-footer} else {set-footer $m}

#   if {$action == 1} then {inform vmecontrol SAC_GO}

   if {$action == 3} then {define_VME_addresses}

   redisplay-vme
}

