# fsysc.tcl---Silena FAIR System Controller module control frame



set frame "FSYSC"
set frametitle "Silena FAIR System Controller module"
set icontitle "FSYSC"

set modulename fsysc
set module 1

set SYSC $modulename#$module

set Broadcast 0x40
set REICond   0x20

set     CXABTS      0x00
set     Reset       0x01
set     Clear       0x02
set     OnLine      0x03
set     OffLine     0x04
set     OpenChain   0x05
set     CloseChain  0x06
set     Echo        0x07
set     Exec        0x08
set     EndExec     0x09
set     Enbl_Seg    0x0A
set     Dsbl_Seg    0x0B
set     UsrCmd      0x0C
set     Read        0x0D
set     Read_AI     0x0E
set     Read_ROM    0x0F
set     Read_Lab    0x10
set     Read_ID     0x11
set     ReadExec    0x12
set     Read_Grp    0x13
set     Read_MD     0x14
set     Read_CMD    0x15
set     ADC         0x16
set     Write       0x17
set     Write_AI    0x18
set     Write_Lab   0x19
set     Set_ID    0x1A
set     WriteExec   0x1B
set     Enbl_Grp    0x1C
set     Dsbl_Grp    0x1D
set     SetClk      0x1E
set     ERCXABTS    0x1F

set ACK 0x06

set TIMEOUT 500

set NetID 0
set NumADCs 0

set DefaultRMOD 0x1f
set BusClock 100

set RAMOffset 0
set RAMDumpSize 256
set RAMAddrInc 0
set RAMdata ""
set RAMlength 0

loadsource common.vme


proc monitor {s} {
#    insert-log "$s"
#    update
   # puts stdout "$s"
}


proc do_enbl_seg {} {
    global ACK Enbl_Seg Broadcast
    monitor "Enable FAIRbus segment"
    send_command [expr $Enbl_Seg + $Broadcast]
    send_data $ACK
    monitor "Done"
}

proc do_openchain_b {} {
    global ACK OpenChain Broadcast
    monitor "Sending OpenChain + Broadcast"
    send_command [expr $OpenChain + $Broadcast]
    send_data $ACK
    monitor "Done"
}

proc do_closechain_b {} {
    global ACK CloseChain Broadcast
    monitor "Sending CloseChain + Broadcast"
    send_command [expr $CloseChain + $Broadcast]
    send_data $ACK
    monitor "Done"
}

proc do_clear_b {} {
    global ACK Clear Broadcast
    monitor "Sending Clear + Broadcast"
    send_command [expr $Clear + $Broadcast]
    send_data $ACK
    monitor "Done"
}

proc do_closechain_c {} {
    global ACK CloseChain REICond

    monitor "Sending CloseChain"
    send_command [expr $CloseChain + $REICond]
    if {[WaitForAck] != 0} then {
	midas-warning "Time Out Error : The module is not responding"
        error 5
    } else {
        send_data $ACK
    }
    wfci 100
    monitor "Done"
}

proc do_sgreo {} {
    global SYSC
    monitor "Setting NETWORK auto-recognition"
    set-SYSC-register $SYSC.STR 0x88
}

proc F_WriteID {id} {
    global Set_ID REICond

    monitor "Assign NetID $id"

    send_command [expr $Set_ID + $REICond]
    wfns 1
    send_data [expr $id >> 8]
    wfci 2
    send_data [expr $id & 255]
    wfci 3

    monitor "Done"
}

proc FAIR_command {Command v} {
    global REICond NetID ACK

    monitor "Sending FAIRBUS command $Command with NetID $NetID"

#    select Slave

    if {$NetID == 0} {
        send_command [expr $Command + $REICond]
    } else {
        send_command $Command
        send_data [expr $NetID >> 8]
        send_data [expr $NetID & 255]
    }
    wfns 1

#    send command

    send_data $v
    wfci 2
    send_data $ACK
    wfci 3

    monitor "Done"
}


proc FAIR_Xcommand {Command} {
    global REICond NetID ACK

    monitor "Sending FAIRBUS command $Command with NetID $NetID"

#    select Slave

    if {$NetID == 0} {
        send_command [expr $Command + $REICond]
    } else {
        send_command $Command
        send_data [expr $NetID >> 8]
        send_data [expr $NetID & 255]
    }
    wfns 1

    send_data $ACK
    wfci 3

    monitor "Done"
}


proc F_SetClk {v} {
    global SetClk

    monitor "Setting Bus Clock to $v"

    FAIR_command $SetClk $v
}

proc F_UsrCmd {v} {
    global UsrCmd

    monitor "Setting User Command $v"

    FAIR_command $UsrCmd $v
}

proc FAIR_write {address arg} {
    global Write Write_AI REICond NetID

    set data {}
    for {set i 0} {$i < [llength $arg]} {incr i} {
       lappend data [format "0x%04x" [lindex $arg $i]]
    }

    monitor "Writing $data to FAIRBUS register at address [format "0x%04x" $address] with NetID $NetID"

    if {[llength $arg] > 1} then {set Command $Write_AI} else {set Command $Write}

#    select Slave

    if {$NetID == 0} {
        send_command [expr $Command + $REICond]
    } else {
        send_command $Command
        send_data [expr $NetID >> 8]
        send_data [expr $NetID & 255]
    }
    wfns 1

#    send address

    send_data [expr $address >> 8]
    wfci 2
    send_data [expr $address & 255]
    wfci 3

#    send data

    for {set i 0} {$i < [llength $arg]} {incr i} {
        send_data [lindex $arg $i]
        wfci 4
    }

    monitor "Done"
}

proc FAIR_write16 {address data} {FAIR_write $address [list [expr $data >> 8] [expr $data & 255]]}
proc FAIR_write8  {address data} {FAIR_write $address [list $data]}


proc Fwrite_SegC_VSN {vsn} {

    monitor "Setting VSN register to $vsn"

#    the VSN is 16 bits at FAIRbus addresses 7DC0,7DC1

    FAIR_write16 0x7dc0 $vsn
}

proc Fwrite_SegC_RMOD {data} {

    monitor "Setting RMOD register to $data"

#    the RMOD register is 8 bits at FAIRbus addresses 7DC4

    FAIR_write8 0x7dc4 $data
}

proc FAIR_read {address length} {
    global Read Read_AI REICond NetID ACK

    monitor "Reading FAIRbus register at address [format "0x%04x" $address]"

    if {$length > 1} then {set Command $Read_AI} else {set Command $Read}

#    select Slave

    if {$NetID == 0} {
        send_command [expr $Command + $REICond]
    } else {
        send_command $Command
        send_data [expr $NetID >> 8]
        send_data [expr $NetID & 255]
    }
    wfns 1

#    set address

    send_data [expr $address >> 8]
    wfci 2
    send_data [expr $address & 255]
    wfci 3

#    read data

    set data {}
    for {set i 0} {$i < $length } {incr i} {
       send_data $ACK
       lappend data [receive_data]
    }

    monitor "Received [format "0x%04x" $data]"

    return $data
}

proc FAIR_read16 {address} {
    return [expr ([lindex [FAIR_read $address 2] 1] << 8) + [lindex [FAIR_read $address 2] 1]]
}

proc FAIR_read8  {address} {
    return [FAIR_read $address 1]
}

proc Fread_SegC_RMOD {} {

    monitor "Reading the RMOD register"

#    the RMOD register is 8 bits at FAIRbus addresses 7DC4

    set data [FAIR_read8 0x7dc4]

    monitor "Received [format "0x%04x" $data]"

    return $data
}

proc Fread_SegC_STR {} {

    monitor "Reading SegC STR register"

#    the STR register is 8 bits at FAIRbus addresses 7DC5

    set data [FAIR_read8 0x7dc5]

    monitor "Received [format "0x%04x" $data]"

    return $data
}

proc wfns {step} {
    if {[WaitForAck] != 0} then {midas-warning "Time Out: No Slave online at step $step"; error 1}
}

proc wfci {step} {
    if {[WaitForAck] != 0} then {midas-warning "Time Out: Connection interrupted at step $step"; error 2}
}


proc send_command {cmd} {Send [expr $cmd + 0x100]}
proc send_data {data}   {Send $data}

proc Send {data} {
    global SYSC
    set-SYSC-register $SYSC.NET $data
    WaitForSend
}

proc received {} {
    global SYSC 
    set s [read-SYSC-register $SYSC.NET]
    if {[expr $s & 0x0200] != 0} then {
        if {[expr $s & 0x0100] != 0} then {return 1} else {return 2}
    } else {
        return 0
    }
}

proc receive_data {} {
    global TIMEOUT
    for {set i 0} {$i < $TIMEOUT} {incr i} {
       set j [received]
       if {$j != 0} then {
           set d [Receive]
           if {$j == 1} then {return [expr $d + 0x100]} else {return $d}
       }
    }
    midas-warning "Timeout receiving FAIRbus data"
    error 3
}
        
proc Receive {} {
    global SYSC 
    return [expr [read-SYSC-register $SYSC.DNET] & 0xff]
}

proc WaitForSend {} {
    global SYSC 
    global TIMEOUT
    for {set i 0} {$i < $TIMEOUT} {incr i} {
        if {[expr [read-SYSC-register $SYSC.NET] &  0x0400] == 0} return
    }
    midas-warning "Timeout sending FAIRbus command"
    error 4
}

proc WaitForAck {} {
    global ACK TIMEOUT

    for {set i 0} {$i < $TIMEOUT} {incr i} {
       if {[received] == 1} then {
           set j [Receive]
           if {$j == $ACK} then {return 0} \
             else {insert-log "received command $j from FAIRbus SysC"; return 1}
       }
       if {[received] == 2} then {
           set j [Receive]
           insert-log "received data $j from FAIRbus SysC"
           return 1
       }
    }
    return 1
}


proc set-SYSC-register {r v} {
    monitor "setting SYSC register $r [format "0x%04x" $v]"
    write-vme-register $r $v
}

proc read-SYSC-register {r} {
    set data [read-vme-register $r]
    monitor "read SYSC register $r - returned [format "0x%04x" $data]"
    return $data
}


#    Initialise FAIRbus System Controller

proc do_initsysc {} {
    global SYSC 

#    do SysC reset - set the CLR bit in the STR register
    monitor "Resetting the FAIR System Controller"
    set-SYSC-register $SYSC.STR 0x28
#    wait 200 millisecs
    after 200
    set-SYSC-register $SYSC.STR 0x08
#    wait 1 sec
    after 1000
    monitor "Done"

#    do FAIR bus reset - unset the RESETBUS bit in the STR register
    monitor "Resetting the FAIR bus"
    set-SYSC-register $SYSC.STR 0x00
#    wait 1 sec
    after 1000
    set-SYSC-register $SYSC.STR 0x08
#    wait 2 sec
    after 2000
    monitor "Done"

#    Init GLINK & TRGWIN
    monitor "Setting SYSC DELR register"
    set-SYSC-register $SYSC.DELR 0x2018
#    Init Threshold Bank 1
    monitor "Setting SYSC THREB1 register"
    set-SYSC-register $SYSC.THREB1 0xf07
#    Init Threshold Bank 2
    monitor "Setting SYSC THREB2 register"
    set-SYSC-register $SYSC.THREB2 0xf07

#    Init Control Register
    monitor "Setting SYSC CTRL register"
    set-SYSC-register $SYSC.CTRL 0

    monitor "Done"
}


#    Initialise FAIRbus Segment Controller

proc do_initsegc {n} {
    global DefaultRMOD BusClock

#    assign a NetID to the SegC

    F_WriteID $n

#    assign a VSN to the SegC  -  The VSN is the same as its NetID

    Fwrite_SegC_VSN $n

#    run user command 0 which sets the VSN parity in the SegC

    F_UsrCmd 0

#    Set the RMOD register being careful to preserve the VSN parity bit
     
    set data [Fread_SegC_RMOD]

    Fwrite_SegC_RMOD [expr ($data & 0x80) + $DefaultRMOD]

#    Set the Bus Clock for this slave 

    F_SetClk $BusClock
}


proc FAIR_Initialise {} {
   global NetID SYSC NumADCs EGreport

   set NetID 0
   set NumADCs 0

   set EGreport_SAVE $EGreport
   set EGreport -1
   set-busy

#    initialise the system controller

   insert-log "Initialise FAIRbus System Controller"
   update
   do_initsysc

#    enable FAIRbus segment

   do_enbl_seg

#    open the FAIR bus chain

   do_openchain_b

#    start AUTO Recognition

   do_sgreo

#    first to respond will be the segment controller
#    initialise the segment controller and assign it a NetID

   insert-log "Initialise FAIRbus Segment Controller"
   update
   do_initsegc 0xff

#    finished with the segment controller - make it pass on the token

   do_closechain_c

#    assign a NetID to each ADC module

   for {set ID 1} {$ID < 255}  {incr ID} {

#    all ADCs have been found when the SYSC SGREI Signal is set

       if {[expr [read-SYSC-register $SYSC.STR] & 0x80]} then {break}

       F_WriteID $ID
       do_closechain_c
       incr NumADCs
       update
   }
   insert-log "Discovered $NumADCs ADC modules"

   do_openchain_b

   set EGreport $EGreport_SAVE
   clear-busy
}

proc FAIR_DAQ-setup {} {
    global NumADCs NetID EGreport

#    perform SETUP for all the ADC modules

   set EGreport_SAVE $EGreport
   set EGreport -1
   set-busy
   
   inform sl :modify LIST :strings [transparent "Performing setup for 9418 module"]
   
   for {set ID 1} {$ID <= $NumADCs} {incr ID} {
       inform sl :modify LIST :strings [transparent " #$ID"]
       update
       set NetID $ID
       FAIR_write8 0x424 0
       FAIR_write8 0x425 $ID
       FAIR_write8 0x422 0x20
       FAIR_write8 0x420 255
       FAIR_write8 0x426 0
       FAIR_write8 0x427 0
       for {set i 0} {$i < 32} {incr i} {
          FAIR_write8 [expr 0x400 + $i] 8
       }
   }
   insert-log " done"

   set EGreport $EGreport_SAVE
   clear-busy
}

proc FAIR_DAQ-go {} {
    global NumADCs NetID SYSC EGreport

#    perform GO for all the ADC modules

   set EGreport_SAVE $EGreport
   set EGreport -1
   set-busy

#   for {set ID 1} {$ID <= $NumADCs} {incr ID} {
#       set NetID $ID
#       FAIR_write8 0x422 0x20
#   }

  do_clear_b

  set-SYSC-register $SYSC.CTRL 0
  set-SYSC-register $SYSC.STR 0x58

   set EGreport $EGreport_SAVE
   clear-busy
}

proc FAIR_DAQ-halt {} {
    global NumADCs NetID SYSC EGreport

#    perform STOP for all the ADC modules

   set EGreport_SAVE $EGreport
   set EGreport -1
   set-busy

  set-SYSC-register $SYSC.STR 0x08

#   for {set ID 1} {$ID <= $NumADCs} {incr ID} {
#       set NetID $ID
#       FAIR_write8 0x422 1
#   }

   set EGreport $EGreport_SAVE
   clear-busy
}



proc click-delr {value} {
    global SYSC 

    set register DELR
    set-SYSC-register $SYSC.$register $value
}

proc click-str {value} {
    global SYSC 

    set register STR
    set-SYSC-register $SYSC.$register $value
}

proc click-acq {value} {
    global SYSC 

    set register STR
    set-SYSC-register $SYSC.$register [expr $value + 2]
}


proc click-redisplay {} {
    global SYSC 

    set register DELR
    set vl [read-SYSC-register $SYSC.$register]
    :modify DELR :value [expr $vl & 255]

    set register STR
    set vl [read-SYSC-register $SYSC.$register]
    :modify STR :value $vl
    :modify ACQ :value [expr $vl & 1]
}

proc start args {
   global started EGreport

   if {$started == 0} then {
       make-frame
       EGbindings
   }
   set started 1
   open-frame

   clear-footer
   clear-busy
   set EGreport -1

  click-redisplay
}

proc click-quit {} {close-frame}

proc finish args {destroy-frame}


proc click arg {
   global debug modulename RAMOffset RAMDumpSize RAMAddrInc SYSC
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {
      ENGINEERING       {
         switch $v  {
            Software   {inform [set modulename]sftwre start}
            Initialise {FAIR_Initialise}
            Setup      {FAIR_DAQ-setup}
            Halt       {FAIR_DAQ-halt}
            Go         {FAIR_DAQ-go}
         }
      }
      DELR              {click-delr $v}
      STR               {click-str $v}
      ACQ               {click-acq $v}

      RAMOFFSET {set RAMOffset $v}

      RAMREAD     {do_readRAM $RAMDumpSize $SYSC.RAM}
      RAMEMPTY     {:modify RAMDISPLAY :empty T}

      RAMINCR      {set RAMAddrInc $v}
      RAMLENGTH    {set RAMDumpSize $v}

      default           {}
   }
}

proc do_readRAM {length register} {
  global CurrentVMEServer
  global RAMOffset RAMAddrInc RAMdata RAMlength

  set len 64

  :modify RAMDISPLAY :strings "\n  RAM Offset $RAMOffset\n"
  set offset $RAMOffset

  set RAMdata ""

  for {set l 0} {$l < $length} {incr l $len} {
      EG "register :name '/$CurrentVMEServer/$register' :object '-o $RAMOffset -l $len'"
      append RAMdata [string trimleft [read-vme-register $register]]
      set RAMOffset [expr $RAMOffset + ($len * 4)]
  }

  set RAMlength $l

  :modify RAMDISPLAY :strings "$RAMdata"

  if {$RAMAddrInc == 0} then {set RAMOffset $offset}
  :modify RAMOFFSET :value $RAMOffset
}

proc format_the_dump {} {
  global RAMdata RAMlength

#    format data held in global variable RAMdata

   set-footer "formatting RAM"

   set frame "ram"
   set win [self frame-name].$frame
   :frame $win :label  "Formatted RAM Dump"  :transient T
   :layout T

        :galley G :rows 50 :cols 50 :deaf T :strings "" :font fixed

        for {set i 0} {$i < $RAMlength} {incr i} {
           set word [lindex $RAMdata $i]

           if {$word == 0xffffffff} {
#    end event
               :modify G :strings "$word\n\n"
               continue
           }
           if {[format 0x%x [expr $word & 0x800000ff]] == 0x800000ff} {
#    9402 footer
               set title "9402 start token"
               set EC [expr $word >> 26 & 15]
               :modify G :strings "[format "%10s ec=%2d %26s" $word $EC $title] \n"
               continue
           }
           if {[format 0x%x [expr $word & 0x80000000]] == 0x80000000} {
#    9418 footer
               set title "9418 footer"
               set VSN [expr $word & 255]
               set EC [expr $word >> 26 & 15]
               set WC [expr $word >> 16 & 0x3f]
               :modify G :strings "[format "%10s ec=%d wc=%2d vsn=%3d %12s" $word $EC $WC $VSN $title] \n"
               continue
           }
           if {[expr $word & 0x80000000] == 0} {
#    adc data
               set title "adc data"
               set channel [expr $word >> 20 & 31]
               set adc [expr $word & 0x0fff]
               :modify G :strings "[format "%10s ch=%3d data=%4x %15s" $word $channel $adc $title] \n"
               continue
           }
           :modify G :strings "$word - ********unknown format********"
        }

   :modify G :see 0
   :layout B; :set fill none; :set anchor center
   :button B  :label "Dismiss" :command "destroy $win"
   :show T
   clear-footer
}

proc make-frame {} {
  global env RAMOffset RAMAddrInc RAMDumpSize

set cols 90
set rows 10

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" 

   :panel MODULE  :layout V :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

     :slider DELR       :label "Trigger Delay"  :min 0 :max 255  :valuex 12c
     :text STR          :label  Status :w 10  :valuex 12c
     :checkbox ACQ      :label "Acquisition On" :valuex 12c

   :panel DISPLAY :layout V :padx 10 
      :layout H; :row-gap 10; :col-gap 10
      :text RAMOFFSET :label "RAM Offset:" :w 6 :value "$RAMOffset"
      :button RAMREAD :label "read"
      :checkbox RAMINCR :label "increment RAM address" :value $RAMAddrInc
      :text RAMLENGTH :label "Length (words)" :value $RAMDumpSize :w 4
      :button RAMEMPTY     :label "empty RAM display"

   :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 30; :row-gap 10
      :vskip 10
      :button REDISPLAY  :command click-redisplay
      :menubutton ENGINEERING :label "Engineering"  :menu [list Software Initialise Setup Halt Go]
      :button FORMAT :label "Format RAM display" :command format_the_dump

    :panel P1 :layout V
      :galley RAMDISPLAY :rows $rows :cols $cols :deaf T :font fixed

  open-frame

  :galley_size RAMDISPLAY $rows $cols
}

proc click-configure {W} {:galley_resize RAMDISPLAY}
